@@ -12,14 +12,19 @@ from .cache import cache_to_disk, cache_to_disk_binary, get_cache_filename
 from .comparer import ResultsComparer
 from .file_io import (
     BinaryFileHandler,
+    GzipFileHandler,
     VersionMismatchError,
     copy_file,
     delete_file,
+    delete_files,
     file_exists,
+    get_files,
+    get_folders,
     get_public_url,
     list_files,
     most_recent_timestamp,
     move_file,
+    purge_folder,
     read_file,
     read_file_with_version,
     update_file_if_version_matches,
@@ -30,14 +35,18 @@ from .storage import JsonFileStorage, json_file_storage_factory
 
 __all__ = [
     "BinaryFileHandler",
+    "GzipFileHandler",
     "cache_to_disk_binary",
     "cache_to_disk",
     "copy_file",
     "delete_file",
+    "delete_files",
     "file_exists",
     "FileLock",
     "get_cache_filename",
     "get_public_url",
+    "get_files",
+    "get_folders",
     "json_file_storage_factory",
     "JsonFileStorage",
     "list_files",
@@ -49,5 +58,6 @@ __all__ = [
     "update_file_if_version_matches",
     "VersionMismatchError",
     "Waiter",
+    "purge_folder",
     "write_file",
 ]
