@@ -57,8 +57,10 @@ class VersionMismatchError(FileNotFoundError):
 def get_bucket_key(path: str) -> Tuple[str, str]:
     """Get the bucket and key from a S3 path"""
     path = path.replace(S3_PREFIX, "")
-    bucket, key = path.split("/", 1)
-    return bucket, key
+    if "/" in path:
+        bucket, key = path.split("/", 1)
+        return bucket, key
+    return path, ""
 
 
 def file_exists(file: str) -> bool:
@@ -184,12 +186,59 @@ def delete_file(file: str) -> None:
             pass
 
 
-def delete_files(files: list[str], max_workers: int = 500) -> None:
-    """Delete multiple files from S3 or local storage, using up to max_workers threads."""
+def _delete_files_s3(files: list[str], max_workers: int = 500) -> None:
+
+    # Split into S3 and local paths
+    s3_by_bucket: dict[str, list[str]] = {}
+
+    for f in files:
+        bucket, key = get_bucket_key(f)
+        if not key:  # pragma: no cover
+            # Skip bucket-only paths
+            continue
+        s3_by_bucket.setdefault(bucket, []).append(key)
+
+    def _s3_delete_chunk(bucket: str, keys_chunk: list[str]) -> None:
+        if not keys_chunk:  # pragma: no cover
+            return
+        # Quiet response to minimize payload; S3 ignores non-existent keys
+        s3_client.delete_objects(
+            Bucket=bucket,
+            Delete={
+                "Objects": [{"Key": k} for k in keys_chunk],
+                "Quiet": True,
+            },
+        )
+
+    # Execute S3 batch deletions and local deletions in parallel
+    with ThreadPoolExecutor(max_workers=max_workers) as executor:
+        # Queue S3 batches
+        for bucket, keys in s3_by_bucket.items():
+            for i in range(0, len(keys), 1000):
+                executor.submit(_s3_delete_chunk, bucket, keys[i : i + 1000])
+
+
+def _delete_files_local(files: list[str], max_workers: int = 500) -> None:
     with ThreadPoolExecutor(max_workers=max_workers) as executor:
         executor.map(delete_file, files)
 
 
+def delete_files(files: list[str], max_workers: int = 500) -> None:
+    """Delete multiple files from S3 or local storage.
+
+    - For S3 paths, uses the `delete_objects` batch API (up to 1000 keys/request)
+      and groups deletions by bucket for efficiency.
+    - For local paths, deletes concurrently using up to `max_workers` threads.
+    """
+    s3_files = [f for f in files if f.startswith(S3_PREFIX)]
+    local_files = [f for f in files if not f.startswith(S3_PREFIX)]
+
+    if s3_files:
+        _delete_files_s3(s3_files, max_workers=max_workers)
+    if local_files:
+        _delete_files_local(local_files, max_workers=max_workers)
+
+
 def copy_file(src: str, dst: str, chunk_size: int = 131072) -> None:
     """Copy a file from S3 or local storage"""
     if src.startswith(S3_PREFIX) and dst.startswith(S3_PREFIX):
@@ -220,32 +269,80 @@ def move_file(src: str, dst: str) -> None:
     delete_file(src)
 
 
-def get_files(root_path: str, prefix: str = "") -> List[str]:
-    """Returns a list of files from S3 or local storage with the relevant prefix.
+def _get_files_s3(
+    root_path: str, prefix: str = "", cutoff_utc: datetime | None = None, cutoff_earlier: bool = True
+) -> List[str]:
+    bucket, key = get_bucket_key(root_path)
+    key = key + "/" if key and not key.endswith("/") else key
 
-    The prefix significantly improves performance for S3 by reducing the number of objects listed.
-    """
-    if root_path.startswith(S3_PREFIX):
-        bucket, key = get_bucket_key(root_path)
-        key = key + "/" if not key.endswith("/") else key
+    paginator = s3_client.get_paginator("list_objects_v2")
 
-        paginator = s3_client.get_paginator("list_objects_v2")
-        files = []
+    # Normalize cutoff_date to timezone-aware UTC for comparison consistency
+    iterator = paginator.paginate(Bucket=bucket, Prefix=key + prefix, PaginationConfig={"PageSize": 10_000})
 
-        for page in paginator.paginate(Bucket=bucket, Prefix=key + prefix, PaginationConfig={"PageSize": 10_000}):
-            if "Contents" in page:
-                files.extend([obj["Key"].removeprefix(key) for obj in page["Contents"]])
+    if cutoff_utc:
+        comparator = "<" if cutoff_earlier else ">="
+        search = (
+            "Contents[?to_string(LastModified)"
+            + comparator
+            + "'\""
+            + cutoff_utc.strftime("%Y-%m-%d %H:%M:%S%z")
+            + "\"'].Key"
+        )
+    else:
+        search = "Contents[].Key"
+    return [file.removeprefix(key) for file in iterator.search(search) if file]
 
-        return files
 
-    root_path = os.path.abspath(root_path)
-    results: List[str] = []
+def _generate_local_files(root_path: str, prefix: str = "") -> Generator[str, None, None]:
+    root_path = os.path.abspath(os.path.normpath(root_path)) + os.path.sep
     for dirpath, _, filenames in os.walk(root_path):
         for name in filenames:
             rel = os.path.relpath(os.path.join(dirpath, name), root_path)
-            if rel.startswith(prefix):
-                results.append(rel)
-    return results
+            if not prefix or rel.startswith(prefix):
+                yield rel
+
+
+def _generate_local_files_cutoff(
+    cutoff_utc: datetime, root_path: str, prefix: str = "", cutoff_earlier: bool = True
+) -> Generator[str, None, None]:
+    # Normalize cutoff_date to timezone-aware UTC for comparison consistency
+    for rel in _generate_local_files(root_path, prefix):
+        full_path = os.path.join(root_path, rel)
+        try:
+            mtime = datetime.fromtimestamp(os.path.getmtime(full_path), tz=timezone.utc)
+        except FileNotFoundError:  # pragma: no cover
+            # File may have been deleted during traversal; skip
+            continue
+        if (mtime < cutoff_utc) == cutoff_earlier:
+            yield rel
+
+
+def get_files(
+    root_path: str, prefix: str = "", cutoff_date: datetime | None = None, cutoff_earlier: bool = True
+) -> List[str]:
+    """Returns a list of files from S3 or local storage with the relevant prefix.
+
+    - If `cutoff_date` is provided, only returns files whose last modified time is strictly
+      earlier or later than `cutoff_date`.
+    - If `cutoff_earlier` is True (default), returns files older than `cutoff_date`,
+      otherwise returns files newer or equal to `cutoff_date`.
+    - The prefix significantly improves performance for S3 by reducing the number of objects listed.
+    """
+    # Normalize cutoff_date to timezone-aware UTC for comparison consistency
+    cutoff_utc: datetime | None
+    if cutoff_date is None:
+        cutoff_utc = None
+    else:
+        cutoff_utc = cutoff_date if cutoff_date.tzinfo else cutoff_date.replace(tzinfo=timezone.utc)
+        cutoff_utc = cutoff_utc.astimezone(timezone.utc)
+
+    if root_path.startswith(S3_PREFIX):
+        return _get_files_s3(root_path, prefix, cutoff_utc, cutoff_earlier)
+
+    if cutoff_utc:
+        return list(_generate_local_files_cutoff(cutoff_utc, root_path, prefix, cutoff_earlier))
+    return list(_generate_local_files(root_path, prefix))
 
 
 def _s3_get_folders(root_path: str, prefix: str = "") -> List[str]:
@@ -385,22 +482,28 @@ class BinaryFileHandler:
                 self.path, self.mode, encoding="UTF-8" if self.mode == "w" else None
             )
 
+    def _set_buffer_http(self) -> None:
+        with _get_response(self.path) as response:
+            self._buffer = BytesIO(response.read())
+        self._buffer.seek(0)
+
+    def _set_buffer_s3(self) -> None:
+        bucket, key = get_bucket_key(self.path)
+        if self.mode == "rb":
+            # Download the file from S3 to the stream
+            try:
+                s3_client.download_fileobj(Bucket=bucket, Key=key, Fileobj=self._buffer)
+            except s3_client.exceptions.ClientError as exc:
+                raise FileNotFoundError(f"File {self.path} not found") from exc
+            self._buffer.seek(0)
+
     def __enter__(self) -> BytesIO | IO:
         """Read from S3, HTTPS, or open the stream"""
         if self.is_https:
-            with _get_response(self.path) as response:
-                self._buffer = BytesIO(response.read())
-            self._buffer.seek(0)
+            self._set_buffer_http()
 
         if self.is_s3:
-            bucket, key = get_bucket_key(self.path)
-            if self.mode == "rb":
-                # Download the file from S3 to the stream
-                try:
-                    s3_client.download_fileobj(Bucket=bucket, Key=key, Fileobj=self._buffer)
-                except s3_client.exceptions.ClientError as exc:
-                    raise FileNotFoundError(f"File {self.path} not found") from exc
-                self._buffer.seek(0)
+            self._set_buffer_s3()
 
         return self._buffer
 
@@ -439,7 +542,12 @@ def GzipFileHandler(path: str, mode: str = "rb", encoding: str | None = None) ->
             yield gz_file
 
 
-def purge_folder(root_path: str, dry_run: bool = True, max_workers: int = 500) -> List[str]:
+def purge_folder(
+    root_path: str,
+    dry_run: bool = True,
+    max_workers: int = 500,
+    cutoff_date: datetime | None = None,
+) -> List[str]:
     """Delete all files within a folder/prefix on S3 or a local directory.
 
     For S3, root_path should be an S3 URL (s3://bucket/path/). Uses get_files to list objects
@@ -449,11 +557,24 @@ def purge_folder(root_path: str, dry_run: bool = True, max_workers: int = 500) -
     :param root_path: The S3 bucket/prefix or local directory to purge
     :param dry_run: If True, no files are deleted, and the function returns the list of files that would be deleted
     :param max_workers: The maximum number of worker threads to use for deletion
+    :param cutoff_date: If provided, only purge files older than this datetime
 
     Returns a sorted list of full paths of files deleted (or that would be deleted).
     """
-    full_paths = sorted(f"{root_path.rstrip('/')}/{f}" for f in get_files(root_path))
+    if root_path.startswith(S3_PREFIX):
+        full_paths = sorted(f"{root_path.rstrip('/')}/{f}" for f in get_files(root_path, cutoff_date=cutoff_date))
+    else:
+        full_paths = sorted(
+            os.path.join(os.path.normpath(root_path), f) for f in get_files(root_path, cutoff_date=cutoff_date)
+        )
 
     if not dry_run:
         delete_files(full_paths, max_workers=max_workers)
+
+        if not root_path.startswith(S3_PREFIX):
+            # Clean up any subdirectories
+            for dirpath, _, _ in os.walk(root_path, topdown=False):
+                if not os.listdir(dirpath):
+                    os.rmdir(dirpath)
+
     return full_paths
